#!/usr/bin/env python
# coding: utf-8

# In[1]:


import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from sklearn import set_config
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import OrdinalEncoder

from sksurv.datasets import load_gbsg2
from sksurv.preprocessing import OneHotEncoder
from sksurv.ensemble import RandomSurvivalForest

class SurvivalForest:
    def __init__(self, x_path, y_path):
        X = pd.read_csv(x_path, dtype={'age':float,
'estrec':       float,
'horTh':       'category',
'menostat':    'category',
'pnodes':       float,
'progrec':      float,
'tgrade':      'category',
'tsize':        float,})
        y = np.genfromtxt(y_path, delimiter=',', dtype='bool, float')
        grade_str = X.loc[:, "tgrade"].astype(object).values[:, np.newaxis]
        grade_num = OrdinalEncoder(categories=[["I", "II", "III"]]).fit_transform(grade_str)

        X_no_grade = X.drop("tgrade", axis=1)
        Xt = OneHotEncoder().fit_transform(X_no_grade)
        Xt.loc[:, "tgrade"] = grade_num
        
        random_state = 20

        self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(
            Xt, y, test_size=0.25, random_state=random_state)
        
        self.rsf = RandomSurvivalForest(n_estimators=1000,
                           min_samples_split=10,
                           min_samples_leaf=15,
                           n_jobs=-1,
                           random_state=random_state)
    def train(self):
        self.rsf.fit(self.X_train, self.y_train)
    
    def score(self):
        return self.rsf.score(self.X_test, self.y_test)
    
    def predict(self, x):
        X = pd.DataFrame([x], columns=['age','estrec','horTh=yes','menostat=Pre','pnodes','progrec','tsize','tgrade'])
        return self.rsf.predict(X)
